<?php
/**
 * Cms_syslog.php
 *
 * System log class. This is the initial implementation and only a small set of functions
 * is avaiable. This needs to be extended when there are more logging requirements.
 *
 *
 * @author $Author: wichai $
 * @version $Id: Cms_syslog.php,v 1.5 2011/10/19 04:13:26 wichai Exp $
 * @copyright Copyright (c) 2011, Tiller Software Co., Ltd.
*/

class Cms_syslog {
	const TYPE_EVALUATION_ARCHIVE = 1;
	private $CI;
	private $have_page_links = FALSE;

	function __construct() {
		$this->CI = & get_instance();
		$this->CI->load->model('cms/syslog_model');
		osa_load_lang('syslog');
	}

	//This is a system class so protect it from lazy coders...
	function __set($var, $value) {
		echo __METHOD__ . " - You are not allowed to set undeclared class property '$var', bye bye.";
		exit;
	}

	//Writes one log entry
	function write($type, $text) {
		if ( !osa_is_int($type) ) {
			return FALSE;
		}
		$userid = NULL;
		$user = osa_login_object();
		if ( $user && isset($user->id) && osa_is_int($user->id) ) {
			$userid = $user->id;
		}
		return $this->CI->syslog_model->insert($type, $text, $userid);
	}

	//Gets the log entries by type with optional userid
	function get($type, $userid=FALSE) {
		if ( !osa_is_int($type) ) {
			return FALSE;
		}
		return $this->CI->syslog_model->get($type, $userid);
	}

	//Gets the log entries by user with optional type
	function get_by_userid($userid, $type=FALSE) {
		if ( !osa_is_int($userid) ) {
			return FALSE;
		}
		return $this->CI->syslog_model->get_by_userid($userid, $type);
	}

	/*
	 * $data - the data set from get() or get_by_userid
	 * $perpage_count - Show all records if set to false otherwise sets the number of entries per page
	 * $show_top_links - Show the top page links or not. Not effect if $perpage_count is set to false
	 * $show_bottom_links - For bottom page links.
	 * $return_html - returns the HTML is set to true or loads the view if false
	 */

	function print_table($data, $perpage_count=FALSE, $show_top_links=TRUE, $show_bottom_links=TRUE, $return_html=FALSE) {
		if ( !is_array($data) ) {
			return FALSE;
		}
		require_once(APPPATH . 'libraries/mapping/display_list.php');
		$display = new display_list();
		$display->reset();
		$display->titles = array(lang('syslog_date_title'), lang('syslog_user_title'), lang('syslog_log_title'));
		$display->widths = array('17%','20%','63%');
		$display->view_file = 'template/syslog_view';
		$display->sort_direction = display_list::SORT_DESC;
		$display->show_top_links = $show_top_links;
		$display->show_bottom_links = $show_bottom_links;
		if ( !osa_is_int($perpage_count) || $perpage_count <=0 ) {
			$display->allpage = TRUE;
		}
		else {
			$display->perpage = $perpage_count;
		}
		// if $data is empty array will ignore this foreach
		// that mean show only title column, no data
		foreach ($data as $record) {
			$realname = osa_user_getname($record);
			$row = array($record->createdate, $realname, $record->text);
			$display->add($row);
		}
		$data = new stdClass();
		$data->display = & $display;
		//syslogdiv is also hardcoded in the view
		$data->refresh_div = 'syslogdiv';
		$display->divid = $data->refresh_div;
		$display->prepare_page();
		if ( !$return_html ) {
			$this->CI->load->view($display->view_file, $data);
		}
		else {
			return $this->CI->load->view($display->view_file, $data, TRUE);
		}
	}

}