<?php
/**
 * cms_exception.php - Extends the PHP Exception class
 *
 * @author $Author: dtong $
 * @version $Id: cms_exception.php,v 1.7 2012/01/25 10:28:44 dtong Exp $
 * @copyright Copyright (c) 2011, Tiller Software Co., Ltd.
*/

class cms_exception extends Exception {

	private static $initialized = FALSE;
	private static $log_flag = TRUE;

	/*
	 * This is our own exception class. It also does loggin by default. The error message retured can
	 * be displayed to the user. The error code is in the exception_code.php file and the language text is
	 * in the excpetionerror_lang.php.
	 *
	 * @param $lang_code string|integer
	 * 	The language code that corresponds to the language index. The language string can be in any language file
	 * 	as long as it is loaded before using cms_exception.
	 * 	Always prepend "exception_code_" to to your code for the language array index.
	 * 	This can be a message for logging purpose. The code detects the presence of a spce in the string
	 * 	and uses it as a message instead of a language code and writes it into the log file,
	 * 	the display message however is same as the one generated for exception without a language code.
	 * @param $status_code integer
	 * 	This can be used as a status code for a method to communicate back to the caller.
	 * 	Default is zero.
	 *
	 * You can pass any number of additional parameters, they will get logged if logging is on.
	 *
	 */
	function __construct($lang_code=FALSE, $status_code=0) {
		if ( !self::$initialized ) {
			osa_load_lang('exceptionerror');
			self::$initialized = TRUE;
		}
		if ( !osa_is_int($status_code) ) {
			$status_code = 0;
		}
		if ( osa_is_ajax() ) {
			$newline1 = '<br />';
			$newline2 = '<br /><br />';
		}
		else {
			$newline1 = "\n";
			$newline2 = "\n\n";
		}
		if ( !$lang_code ) {
			$message = lang('exception_file') . basename($this->getFile()) . $newline1 .
						  lang('exception_line') . $this->getLine();
		}
		elseif ( strpos($lang_code, ' ') ) {
			// If we have a space then we know it is not a lang code and it is a message for logging.
			// This is never show to the user.
			$log_message = $lang_code;
			// Display message if needed.
			$message = lang('exception_file') . basename($this->getFile()) . $newline2 .
						  lang('exception_line') . $this->getLine();
		}
		else {
			$message = '(' . lang('exception_code_title') . ' ' . $lang_code . "){$newline1}" .
						  lang('exception_code_' . $lang_code);
		}
		$display_message = lang('exception_error_occured') . $newline2 . $message;
		parent::__construct($display_message, $status_code);

		// Do logging.
		if ( self::$log_flag ) {
			$args = func_get_args();
			$method = $this->getFile() . ',Line' . $this->getLine() . "\n";
			$trace = "Trace: " . $this->getTraceAsString();
			if ( isset($log_message) ) {
				// Use caller supplied log message if there is one.
				$message = $log_message;
			}
			if ( count($args) > 2 ) {
				array_shift($args);
				array_shift($args);
				osa_log($method, $message, $args, $trace);
			}
			else {
				osa_log($method, $message, $trace);
			}
		}
		else {
			// Turn logging back on in case we have another exception afterward.
			self::$log_flag = TRUE;
		}
	}

	function __toString() {
		return $this->getMessage();
	}

	// Turns off logging for one exception. The object will turn logging back on at the end of every construct function.
	static function log($value) {
		if ( $value ) {
			self::$log_flag = TRUE;
		}
		else {
			self::$log_flag = FALSE;
		}
	}
}
