<?php
/**
 * survey_login_class.php - Class for survey login.
 *
 * @author $Author: dtong $
 * @version $Id: survey_login_class.php,v 1.2 2011/08/08 09:58:30 dtong Exp $
 * @copyright Copyright (c) 2011, Tiller Software Co., Ltd.
*/

class survey_login_class {
	//If userid is set then it means this is an internal CMS user
	public $userid = NULL;
	//The rest are for non-CMS users but valid ldap users
	//All are optional except ext_username. Same as column names from table survey_result
	public $ext_username = NULL;
	public $ext_firstname = NULL;
	public $ext_lastname = NULL;
	public $ext_email = NULL;
	public $ip = NULL;
	public $resultid = NULL; //For result update access check
	private $surveyid = NULL; //Keeping track the current survey id
	//public $orig_surveyid = NULL; //not used
	public $parity = NULL;
	//public $noauth = TRUE; //This flag tell no authentication was performed. Needed for surveyes require no login
	private $completed_surveys = array();
	private $surveycode = NULL;
	private $surveyrecord = NULL;
	private $available = TRUE;

	function __construct() {
		$this->ip = $_SERVER['REMOTE_ADDR'];
	}

	function authenticated() {
		return ($this->userid || $this->ext_username);
	}

	function no_current_survey() {
		return is_null($this->surveyid);
	}

	function get_surveycode() {
		return $this->surveycode;
	}

	function get_surveyrecord() {
		return $this->surveyrecord;
	}

	function get_surveyid() {
		return $this->surveyid;
	}

	function is_available() {
		return $this->available;
	}

	function set_survey($surveycode, $surveyid, $parity, $surveyrecord, $check_available=TRUE) {
		/*
		 * We can assume if $surveyid is set then the rest of the parameters are also set properly.
		 *	$surveyid and $parity come form $surveycode and $surveyrecord come from $surveyid.
		 * So, having $surveyid set means we have the correct surveycode and surveyid points to a valid survey record
		*/
		if ( !osa_is_int_one($surveyid) ) {
			return FALSE;
		}
		if ( $check_available ) {
			if ( $this->available ) {
				$this->available = FALSE;
			}
			else {
				return FALSE;
			}
		}
		$this->surveyid = $surveyid;
		$this->surveycode = $surveycode;
		$this->parity = $parity;
		$this->surveyrecord = $surveyrecord;
		return TRUE;
	}

	function set_completed() {
		if ( !osa_is_int_one($this->surveyid) ) {
			return FALSE;
		}
		$this->_add_complete();
		$this->resultid = NULL;
		//$this->_reset_survey();
		$this->available = TRUE;
		return TRUE;
	}

	function get_completed($surveycode) {
		if ( !array_key_exists($surveycode, $this->completed_surveys) ) {
			return FALSE;
		}
		return $this->completed_surveys[$surveycode];
	}

	private function _add_complete() {
		$this->completed_surveys[$this->surveycode] =
			array('surveyid' => $this->surveyid, 'parity' => $this->parity,
					'resultid' => $this->resultid, 'surveyrecord' => $this->surveyrecord);
	}

	private function _reset_survey() {
		$this->surveyid = NULL;
		$this->surveycode = NULL;
		$this->parity = NULL;
		$this->surveyrecord = NULL;
	}
}