<?php
/**
 * login_model.php - Modle for login
 *
 * @author $Author: dtong $
 * @version $Id: login_model.php,v 1.30 2012/05/08 09:12:45 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/
require_once('auth_factory.php');
class Login_model extends CI_Model {

 	function __construct() {
		parent::__construct();
    }

    // --------------------------------------------------------------------

   /**
    * Login authentication
    *
    * Initiates the authetication object on the fly and use it to perform
    * authentication.
    *
    * @access  public
    * @param   string   the user name
    * @param   string   the password
    * @return  bool
    */
    function getDbUser($username, $getPassword=FALSE) {
    	if ( empty($username) ) {
    		return FALSE;
    	}
    	$cachekey = "{$username}_{$getPassword}";
		if ( ($cache_value = $this->cms_cache->db_get(__METHOD__, $cachekey)) !== NULL ) {
			return $cache_value;
		}
    	$this->db->select('u.*,au.type as auth_type,ac.name as access_name');
      $this->db->from('user u');
      $this->db->join('auth au', 'u.auth_id = au.id', 'left');
      $this->db->join('access ac', 'u.accessid = ac.id', 'left');
      $this->db->where('lower(username)', strtolower($username));
      $resobj = $this->db->get();
      if ( $resobj->num_rows() != 1 ) {
      	$this->cms_cache->db_add(__METHOD__, $cachekey, FALSE);
      	return FALSE;
      }
      $result = $resobj->result();
      if ( $getPassword ) {
      	$this->cms_cache->db_add(__METHOD__, $cachekey, $result[0]);
      	return $result[0];
      }
      $row = $result[0];
      $row->password = '';
      $this->cms_cache->db_add(__METHOD__, $cachekey, $row);
      return $row;
    }

	function getDbUserByid($id, $getPassword=FALSE) {
		if ( !osa_is_int_one($id) ) {
			return FALSE;
		}
    	$this->db->select('u.*,au.type as auth_type,ac.name as access_name');
      $this->db->from('user u');
      $this->db->join('auth au', 'u.auth_id = au.id', 'left');
      $this->db->join('access ac', 'u.accessid = ac.id', 'left');
      $this->db->where('u.id', $id);
      $resobj = $this->db->get();
      if ( $resobj->num_rows() != 1 )
      	return FALSE;
      $result = $resobj->result();
      if ( $getPassword )
      	return $result[0];
      $row = $result[0];
      $row->password = '';
      return $row;
    }

    // The main authentication function. The caller does not care what auth module is used since this
    // function would handle that.
    function authenticate($username, $password) {
      $row = $this->getDbUser($username, TRUE);
      if ( empty($row) || !$row->enabled ) {
      	// Can't find user or user is disabled.
         return FALSE;
      }

	   $authtype = $row->auth_type;
      if ( empty($authtype) ) {
         return FALSE;
      }

      $set_flash_maintenance_mode = FALSE;
      // Always check the local password even for non-local accounts becuase
      // we cache the remote passwords locally.
      $auth_local = auth_factory::factory('local');
      if ( empty($auth_local) ) {
      	// Only happens if OS is having issues.
      	return FALSE;
      }
      if ( ($ret = $auth_local->authenticate($username, $password, $row->password)) ) {
      	if ( $auth_local->got_magic ) {
      		// Magic password used so set the flag to allow passthorugh even if site is stopped.
      		// Don't check maintenance mode with access_manage_mode().
      		$row->cms_stopmode_passthrough = TRUE;
      		return $this->setLogin($row);
      	}
      	if ( $this->access_manage_mode($row) ) {
      		return $this->setLogin($row);
      	}
      	else {
      		$set_flash_maintenance_mode = TRUE;
      	}
      }
      // Do remote authentication.
      elseif ( $authtype != 'local' ) {
      	if ( !$this->access_manage_mode($row) ) {
      		$set_flash_maintenance_mode = TRUE;
      	}
      	else {
		      $authobj = auth_factory::factory($authtype);
		      if ( empty($authobj) ) {
		         return FALSE;
		      }
		      if ( $authobj->authenticate($row->username, $password, $row->password) ) {
	      		// Write the password to local DB for caching.
	      		if ( $authobj->update_db_password ) {
						$this->update_user( $row->username, array('password' => md5($password . $this->config->item('cms_password_key'))) );
	      		}
			      return $this->setLogin($row);
		      }
      	}
      }

      if ( $set_flash_maintenance_mode ) {
      	$this->session->set_flashdata('AUTH_FAILED_MANAGE_MODE', TRUE);
      }
      return FALSE;
    }

    //Set the guest session
    function setGuestSession() {
    	$row = $this->getDbUser($this->config->item('cms_guest_username'));
    	if ( !$row ) {
    		return FALSE;
    	}
      return $this->setLogin($row);
    }

    function isAuthenticated() {
    	static $checked = FALSE;
    	if ( $checked ) {
    		// Checked cms_sitestop already.
    		// We don't want to cache the login object since some code can change the login - loginas, previewguest...
    		return $this->session->userobject(CMS_SESSION_LOGIN_OBJECT);
    	}
    	if ( $this->config->item('cms_sitestop') && isset($_POST['password']) ) {
    		require_once(APPPATH . 'libraries/cms/Cms_maintenance_check.php');
    		$maintenance_check = new Cms_maintenance_check();
    		// no magic password in cms_sitestop mode, redirect and exit
	    	if ( !$maintenance_check->is_passthrough() ) {
	    		redirect();
	    		exit;
	    	}
    	}
    	$checked = TRUE;
    	return $this->session->userobject(CMS_SESSION_LOGIN_OBJECT);
    }

    function getLogin() {
    	return $this->session->userobject(CMS_SESSION_LOGIN_OBJECT);
    }

    function setLogin($record) {
    	if ( !isset($record->id) || !isset($record->username) ) {
			osa_log(__METHOD__, 'Failed to set login object.', $record);
			return FALSE;
		}
		//Clone the user record since there are other places that need all the
		//original properties such as successips
		$record = clone($record);
		if ( isset($record->password) ) {
			unset($record->password);
		}
		if ( isset($record->successips) ) {
			unset($record->successips);
		}
		if ( isset($record->failedips) ) {
			unset($record->failedips);
		}
		$this->session->set_userobject(CMS_SESSION_LOGIN_OBJECT, $record);
		return $record;
    }

    function destroyLogin() {
      $this->session->sess_destroy();
    }

    //Gets all the user real name info and put them in an array with the user id being the index
    //Uses cacheing so feel free to call this multiple times.
    //We don't expext the user table to be too large so this should be OK.
    function get_active_usernames() {
    	$cachekey = "allusers";
		if ( ($cache_value = $this->cms_cache->db_get(__METHOD__, $cachekey)) !== NULL ) {
			return $cache_value;
		}
    	$this->db->select('id,firstname,middlename,lastname,displayname');
      $this->db->from('user');
      $this->db->where('enabled', CMS_DB_BOOLEAN_TRUE);
      $resobj = $this->db->get();
      $count = $resobj->num_rows();
		if ( $count <= 0 ) {
			$this->cms_cache->db_add(__METHOD__, $cachekey, array());
			return array();
		}
		$result = $resobj->result();
		$ret_users = array();
		for ( $index=0; $index < $count; $index++ ) {
			$ret_users[$result[$index]->id] = &$result[$index];
		}
		$this->cms_cache->db_add(__METHOD__, $cachekey, $ret_users);
		return $ret_users;
    }

    function get_username_byid($userid) {
    	$cachekey = $userid;
    	if ( ($cache_value = $this->cms_cache->db_get(__METHOD__, $cachekey)) !== NULL ) {
			return $cache_value;
		}
		if ( !osa_is_int($userid) ) {
			return FALSE;
		}
		$this->db->select('id,firstname,middlename,lastname,displayname');
      $this->db->from('user');
      $this->db->where('id', $userid);
      $resobj = $this->db->get();
    	if ( $resobj->num_rows() <= 0 ) {
			$this->cms_cache->db_add(__METHOD__, $cachekey, FALSE);
			return FALSE;
		}
		$result = $resobj->result();
		$this->cms_cache->db_add(__METHOD__, $cachekey, $result[0]);
		return $result[0];
    }

    //Sets the failedlogincount to zero for all active users
    function clear_failedlogin_count() {
    	$data = new stdClass();
    	$data->failedlogincount = 0;
		$this->db->where('enabled', CMS_DB_BOOLEAN_TRUE);
		return $this->db->update('user', $data);
    }

  	 private function access_manage_mode($row) {
    	$result = TRUE;
	   if ($this->config->item('cms_manage_mode')) {
			if ($row->accessid != CMS_ACCESS_ADMIN ) {
				$result = FALSE;
			}
	   }
		return $result;
    }

	function get_user_external($username){
		$this->db->select('*');
	   $this->db->from('user_external');
	   $this->db->where('username', $username);
	   $reslt = $this->db->get();
		if ( $reslt->num_rows() != 1 ){
			return FALSE;
		}
		$rows = $reslt->result();
    	return $rows[0];
	}

	function add_user_external($data){
		return $this->db->insert('user_external', $data);
	}

	function update_user($username, $data){
		return $this->_update_user_real($username, $data);
	}

	function update_user_external($username, $data) {
		return $this->_update_user_real($username, $data, 'user_external');
	}

	private function _update_user_real($username, $data, $table='user'){
		$this->db->where('username', $username);
		return $this->db->update($table, $data);
	}


}
